<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### Table of Contents

- [Test Waiter Manager][1]
  - [register][2]
  - [unregister][3]
  - [getWaiters][4]
  - [getPendingWaiterState][5]
  - [hasPendingWaiters][6]
  - [\_reset][7]
- [Test Waiter class][8]
  - [TestWaiter][9]
  - [beginAsync][10]
  - [endAsync][11]
  - [waitUntil][12]
  - [debugInfo][13]
- [waitForPromise][14]
  - [Parameters][15]
  - [Examples][16]
- [buildWaiter][17]
  - [Parameters][18]
  - [Examples][19]

## Test Waiter Manager

### register

Registers a waiter.

#### Parameters

- `waiter` {IWaiter} A test waiter instance

### unregister

Unregisters a waiter.

#### Parameters

- `waiter` {IWaiter} A test waiter instance

### getWaiters

Gets an array of all waiters current registered.

Returns **[Array][20]&lt;IWaiter>**

### getPendingWaiterState

Gets the current state of all waiters. Any waiters whose
`waitUntil` method returns false will be considered `pending`.

Returns **IPendingWaiterState** An object containing a count of all waiters
pending and a `waiters` object containing the name of all pending waiters
and their debug info.

### hasPendingWaiters

Determines if there are any pending waiters.

Returns **[boolean][21]** `true` if there are pending waiters, otherwise `false`.

### \_reset

Clears all waiters.

## Test Waiter class

### TestWaiter

A class providing creation, registration and async waiting functionality.

#### Parameters

- `name`

### beginAsync

Should be used to signal the beginning of an async operation that
is to be waited for. Invocation of this method should be paired with a subsequent
`endAsync` call to indicate to the waiter system that the async operation is completed.

#### Parameters

- `item` {T} The item to register for waiting
- `label` {string} An optional label to identify the item

### endAsync

Should be used to signal the end of an async operation. Invocation of this
method should be paired with a preceeding `beginAsync` call, which would indicate the
beginning of an async operation.

#### Parameters

- `item` {T} The item to that was registered for waiting

### waitUntil

Used to determine if the waiter system should still wait for async
operations to complete.

Returns **[boolean][21]**

### debugInfo

Returns the `debugInfo` for each item tracking async operations in this waiter.

Returns **ITestWaiterDebugInfo**

## waitForPromise

A convenient utility function to simplify waiting for a promise.

### Parameters

- `promise` {Promise<T>} The promise to track async operations for
- `label` {string} An optional string to identify the promise

### Examples

```javascript
import Component from '@ember/component';
import { waitForPromise } from 'ember-test-waiters';

export default class Friendz extends Component {
  didInsertElement() {
    waitForPromise(
      new Promise(resolve => {
        doSomeWork();
        resolve();
      })
    );
  }
}
```

## buildWaiter

Builds and returns a test waiter. The type of the
returned waiter is dependent on whether the app or
addon is in `DEBUG` mode or not.

### Parameters

- `name` {string} The name of the test waiter

### Examples

```javascript
import Component from '@ember/component';
import { buildWaiter } from 'ember-test-waiters';

if (DEBUG) {
  let waiter = buildWaiter('friend-waiter');
}

export default class Friendz extends Component {
  didInsertElement() {
    waiter.beginAsync(this);

    someAsyncWork().then(() => {
      waiter.endAsync(this);
    });
  }
}
```

Returns **ITestWaiter**

[1]: #test-waiter-manager
[2]: #register
[3]: #unregister
[4]: #getwaiters
[5]: #getpendingwaiterstate
[6]: #haspendingwaiters
[7]: #_reset
[8]: #test-waiter-class
[9]: #testwaiter
[10]: #beginasync
[11]: #endasync
[12]: #waituntil
[13]: #debuginfo
[14]: #waitforpromise
[15]: #parameters
[16]: #examples
[17]: #buildwaiter
[18]: #parameters-1
[19]: #examples-1
[20]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array
[21]: https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean
